from werkzeug.exceptions import Unauthorized


class UnauthorizedWWWAuthenticate(Unauthorized):
    """
    This is a near verbatim copy of an improvement to an as-yet-unreleased upstream version of werkzeug that allows
    us to specify a www_authenticate argument containing the contents of that field. We should get rid of this once
    we're able to upgrade past that version.

    From werkzeug 8ed5b3f9a285eca756c3ab33f8c370a88eab3842
    """
    def __init__(self, www_authenticate=None, description=None):
        super().__init__(description=description)
        if not isinstance(www_authenticate, (tuple, list)):
            www_authenticate = (www_authenticate,)
        self.www_authenticate = www_authenticate

    def get_headers(self, environ=None):
        headers = super().get_headers(environ)
        if self.www_authenticate:
            headers.append((
                'WWW-Authenticate',
                ', '.join([str(x) for x in self.www_authenticate])
            ))
        return headers



from pathlib import PurePath
import re

from werkzeug.routing import BaseConverter, ValidationError


class SafePurePathConverter(BaseConverter):
    """
        Like the default :class:`PathConverter`, but it converts
        the output to a :class:`pathlib.PurePath` and ensures it
        contains no path components consisting only of dots.
    """

    regex = "[^/].*?"
    weight = 200

    _disallowed_path_part_re = re.compile(r"\.+$")

    def to_python(self, value):
        pth = PurePath(value)
        if not pth.parts:
            raise ValidationError("All path parts squashed out")

        for part in pth.parts:
            if self._disallowed_path_part_re.match(part):
                raise ValidationError(f"Disallowed path part {part!r}")

        return pth
